;; Antoun Kanawati
;; June 20, 1990
;;
;; Revision: June 21/1990 bug fix: references to f000:0000 are in error,
;;           I should use ffff:0000
;;
;; Turbo Assembler source for software rebooting of programs on Turbo XT 3.01
;; BIOS.
;; Technical information from Porgramming Lang conference (Channel 1),
;; and some debugging/disassembly work.
;;
;; The usual DOS reboot is done by calling INT 19h, after setting the word
;; at 40h:72h to either 0 [cold], or 1234h [warm].
;; On my machine [Turbo-XT], this hung up the machine after starting the 
;; boot.  It seems that this a very warm boot, since it skips more than the
;; memory test.
;;
;; The next usual DOS options is: set the word at 40h:72h, then jump to
;; ffffh:0000h.  This worked, but it always gave a cold boot.
;;
;; Using DEBUG, I went and disassembled the code at ffffh:0000h, and found
;; out that all it does is jump to another place.  At this other place, the
;; word 40h:72h is cleared, and then control resumes at f000h:e066h.
;; So, I tried managing 40h:72h on my own, and then jumping to f000h:e066h,
;; and this works fine: the memory test [very fast on my machine] is skipped 
;; when I store 1234h at the magic location; otherwise, a cold boot is
;; executed.
;;
;; The following code demonstrates the implementation.
;;
;; If you want to include this in a library, make sure that all the segments
;; are set up correctly for the memory model you are using.
;; This standalone program is in the Tiny model.
;;
_text	segment	byte public 'code'
	assume	cs:_text, ds:_text, ss:_text

	org	100h    ;; setup for .COM files.
start:	mov	ax, cs
	mov	ds, ax
	mov	ss, ax
;;main procedure
main:
	call	getyn
	mov	bx, 0040h  ;; Note: we are changing DS.
	mov	ds,bx
	or	ax,ax
	jnz	warm
	jmp	far ptr cold

;  	  do it yourself setup [costs 4 extra bytes in code].
;	mov	word ptr ds:[0072h], 0
;	jmp	resume
;;;;;;;;;;;;;;;;;

warm:	mov	word ptr ds:[0072h], 1234h
resume: jmp	far ptr boot ; [i.e.: jmp f000:e066]

;; Loop until user hits Y/y/N/n
getyn:
	mov	ah, 09h
	mov	dx, offset(msg)
	int	21h
_loop:	mov	ah, 08h
	int	21h
	or	al, 20h ;; turn to lower case
	cmp	al, 'y'
	jnz	next
;	mov	ax, 1 ; ax is already non-zero.
	ret
next:	cmp	al, 'n'
	jnz	_loop
	xor	ax, ax
	ret

msg:	db	'Warm boot [Y/N] ?$'
_text	ends

;; fake segment to make BIOS entry points addressable with symbolic
;; labels.
_bios1	segment at 0ffffh
	assume cs:_bios1
	org	00000h
cold:			;; entry here causes a cold boot.
_bios1	ends

_bios2	segment at 0f000h
	assume	cs:_bios2
	org	0e066h
boot:			;; entry here is cold/warm depending on
			;; state of 40h:72h.
_bios2	ends
	end	start
